// FRM_READ.PRG
// Reads the settings in FRM files
// -----------------------------------------------------------------------
// written by Larry Bauder CIS 71320,2707
// 
// contains code "borrowed" from RLBACK.PRG (see note below)
// which carrys a Computer Associates International, Inc. copyright 
// -----------------------------------------------------------------------
// So who uses FRM files any more?  If you do, you may want to store this
// code away for the next time you're trying to track down a setting that
// seems to be backward.  If you have to check a bunch of files, this
// should save you opening all the possible files with RL.EXE.

// This little utility reads a directory of FRM files and reports on certain
// settings found in each file.  The ones that were of interest to me were
// the PageEject, Width and DoubleSpace settings.  With a little work, you
// should be able to add other settings by studying the rlback.prg file
// in the \c52\source\rl\ directory.

// If you do not pass a parameter to the EXE, all FRM files in the current
// directory are processed.  By passing a parameter, you can alter this
// behaviour.  As an example, FRM_READ f:\test\s  will display the settings
// in all FRM files that begin with the letter "s" in the \test directory
// of drive F:.

// A file by the name frm_.txt is created each time you run this utility.
// If a file by that name exists in the current directory, it is overwritten 
// without warning.  Not a big loss!  Unless you use frm_.txt to store 
// something of value in your system.

// This code borrows heavily from rlback.prg as provided by Computer 
// Associates with each copy of Clipper.

// The major change from the RLBACK.prg is the use of the preprocessor
// to declare some global values..

#define size_file_buff         1990   // size of report file.
#define params_offset          1965   // start of report parameters block.
#define size_params_buff       24     // etc. See the rlback.prg for more
#define pe_offset              23     // details
#define page_width_offset      11
#define plnpg_peap_pebp_offset 24
#define dbl_space_offset       21

// and some reassignments to a local array

#define frm_pe           a_private[1]
#define frm_pebp         a_private[2]
#define frm_peap         a_private[3]
#define frm_plainpage    a_private[4]
#define handle           a_private[5]
#define report_file      a_private[6]
#define file_error       a_private[7]
#define read_count       a_private[8]
#define params_buff      a_private[9]
#define plus_byte        a_private[10]
#define frm_pagewidth    a_private[11]
#define frm_dblspaced    a_private[12]
#define mdir             a_private[13]
#define alt_file         a_private[14]
#define MAX_PRIVATE      14

*-------------------------
func main( dir_ )
*-------------------------
local a_temp := {}, a_private[ MAX_PRIVATE ], n, max, file_buff

mdir     := ""            
alt_file := "frm__.txt"
      
if dir_ <> NIL   // and we never refer back to our passed parameter again
  mdir := dir_
endif

a_temp := directory( mdir+"*.frm" ) 

if empty( a_temp )
  ? "No files found"
  inkey( 5 )
  quit
endif

if !("\" $ mdir) .or. right(mdir,1) <> "\"
  if ("\" $ mdir) 
    mdir := substr( mdir, 1, rat("\",mdir)-1 )
  else
    mdir := ""
  endif
endif

if len( a_temp ) > 1
  // sort the array containing our frm file names
  asort( a_temp,1,NIL, {| x, y | x[1] < y[1] } )
endif

file_buff := space( size_file_buff )
max       := len( a_temp )

set alte to (alt_file )
set alte on

? "Form files in "+iif( empty( mdir ), curdir(), mdir )
?

for n := 1 to max

  report_file   := mdir + a_temp[n][1]
  handle        := fopen( report_file )
  file_error    := ferror()

  IF file_error = 0
    // file opening did not have an error, so set up our defaults and a buffer
    frm_pe        := "N"
    frm_pebp      := "Y"
    frm_peap      := "N"
    frm_plainpage := "N"
    frm_dblspaced := "N"
    file_buff     := space( size_file_buff )

    // Read entire file into buffer
    read_count := fread( handle, @file_buff, size_file_buff )

    if read_count = 0
      file_error := -3         // file is empty.
    else
      file_error := ferror()   // check for dos errors
    endif
  else

    ? substr(a_temp[n][1]+space(15),1,13), "File Error", file_error
    loop
  endif
      
  if !fclose( handle )       // we're done with this file so close it
    file_error := ferror()   // we're not checking for errors but we could
  endif                      // 

  // here's the stuff from RLBACK.PRG
  params_buff   := substr( file_buff, params_offset, size_params_buff )
  frm_pe        := substr( params_buff, pe_offset, 1 )
  frm_dblspaced := substr( params_buff, dbl_space_offset, 1 )

  ** process packed 'plus byte' **
  plus_byte := asc(substr(params_buff, plnpg_peap_pebp_offset, 1))
  if int( plus_byte / 4 ) = 1
    frm_plainpage := "Y"
    plus_byte -= 4
  endif
  if int( plus_byte / 2 ) = 1
    frm_peap := "Y"
    plus_byte -= 2
  endif
  if int( plus_byte / 1 ) = 1
    frm_pebp := "N"
    plus_byte -= 1
  endif

  frm_pagewidth := word_2_num( substr( params_buff, page_width_offset, 2 ) )

  // end of stuff from RLBACK.PRG

  ? substr(a_temp[n][1]+space(15),1,13), "PE="+frm_pe, "EJ_BEF="+frm_pebp,;
    "EJ_AFT="+frm_peap, "WIDTH="+str( frm_pagewidth,3,0), "DBLSP="+frm_dblspaced
  
next

?
? "End of Report", date(), time()

set alte off
set alte to
return NIL

*-------------------------
func word_2_num( byte_str )
*-------------------------
// this was also supplied with the code for RL
return( asc( substr( byte_str, 1, 1)) + asc( substr( byte_str, 2, 1)) * 256 )
